/*
 * Copyright (C) 2006-2012 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file error.h List of error messages for all subsystems. */

/** \cond errors */

/* List of all subsystems that use paraslash's error facility. */
DEFINE_ERRLIST_OBJECT_ENUM;

/* these do not need error handling (yet) */
#define SERVER_ERRORS
#define TIME_ERRORS
#define CLOSE_ON_FORK_ERRORS
#define DAEMON_ERRORS
#define GUI_ERRORS
#define GUI_THEME_ERRORS
#define RINGBUFFER_ERRORS
#define SCORE_ERRORS
#define RBTREE_ERRORS
#define RECV_ERRORS
#define IPC_ERRORS
#define DCCP_SEND_ERRORS
#define HTTP_SEND_ERRORS
#define GGO_ERRORS
#define COLOR_ERRORS
#define SIGNAL_ERRORS
#define FADE_ERRORS
#define STDOUT_ERRORS
#define FILE_WRITE_ERRORS
#define STDIN_ERRORS
#define WRITE_ERRORS

extern const char **para_errlist[];

#define OSS_MIX_ERRORS \
	PARA_ERROR(OSS_MIXER_CHANNEL, "invalid mixer channel"), \

#define ALSA_MIX_ERRORS \
	PARA_ERROR(ALSA_MIX_OPEN, "could not open mixer"), \
	PARA_ERROR(ALSA_MIX_BAD_ELEM, "invalid/unsupported control element"), \
	PARA_ERROR(ALSA_MIX_GET_VAL, "could not read control element state"), \
	PARA_ERROR(ALSA_MIX_SET_VAL, "could not set control element state"), \

#define SIDEBAND_ERRORS \
	PARA_ERROR(BAD_BAND, "invalid or unexpected band designator"), \
	PARA_ERROR(SB_PACKET_SIZE, "invalid sideband packet size or protocol error"), \


#define FLACDEC_FILTER_ERRORS \
	PARA_ERROR(FLACDEC_DECODER_ALLOC, "could not allocate stream decoder"), \
	PARA_ERROR(FLACDEC_DECODER_INIT, "could not init stream decoder"), \
	PARA_ERROR(FLACDEC_EOF, "flacdec encountered end of file condition"), \


#define FLAC_AFH_ERRORS \
	PARA_ERROR(FLAC_CHAIN_ALLOC, "could not create metadata chain"), \
	PARA_ERROR(FLAC_CHAIN_READ, "could not read meta chain"), \
	PARA_ERROR(FLAC_ITER_ALLOC, "could not allocate meta iterator"), \
	PARA_ERROR(FLAC_VARBLOCK, "variable blocksize not supported"), \
	PARA_ERROR(FLAC_AFH_DECODER_ALLOC, "could not allocate stream decoder"), \
	PARA_ERROR(FLAC_AFH_DECODER_INIT, "could not init stream decoder"), \
	PARA_ERROR(FLAC_SKIP_META, "could not skip metadata"), \
	PARA_ERROR(FLAC_DECODE_POS, "could not get decode position"), \
	PARA_ERROR(FLAC_STREAMINFO, "could not read stream info meta block"), \


#define OGG_AFH_COMMON_ERRORS \
	PARA_ERROR(STREAM_PACKETOUT, "ogg stream packet-out error (first packet)"), \
	PARA_ERROR(SYNC_PAGEOUT, "ogg sync page-out error (no ogg file?)"), \
	PARA_ERROR(STREAM_PAGEIN, "ogg stream page-in error (first page)"), \
	PARA_ERROR(OGG_SYNC, "internal ogg storage overflow"), \
	PARA_ERROR(OGG_EMPTY, "no ogg pages found"), \


#define SPX_AFH_ERRORS \
	PARA_ERROR(SPX_COMMENT, "invalid speex comment"), \


#define SPX_COMMON_ERRORS \
	PARA_ERROR(SPX_HEADER, "can not read speex header"), \
	PARA_ERROR(SPX_HEADER_MODE, "invalid speex mode in header"), \
	PARA_ERROR(SPX_VERSION, "incompatible speex bit stream version"), \
	PARA_ERROR(SPX_DECODER_INIT, "speex decoder initialization failed"), \
	PARA_ERROR(SPX_CTL_BAD_RQ, "speex_decoder_ctl: invalid request"), \
	PARA_ERROR(SPX_CTL_INVAL, "speex_decoder_ctl: invalid argument"), \

#define SPXDEC_FILTER_ERRORS \
	PARA_ERROR(SPX_DECODE, "speex decoding error"), \
	PARA_ERROR(SPX_DECODE_OVERFLOW, "speex decoding overflow"), \
	PARA_ERROR(SPX_EOS, "speex: end of stream"), \

#define BUFFER_TREE_ERRORS \
	PARA_ERROR(BTR_EOF, "buffer tree: end of file"), \
	PARA_ERROR(BTR_NO_CHILD, "btr node has no children"), \
	PARA_ERROR(BTR_NAVAIL, "btr node: value currently unavailable"), \


#define BITSTREAM_ERRORS \
	PARA_ERROR(VLC, "invalid vlc code"), \


#define WMA_AFH_ERRORS \
	PARA_ERROR(NO_WMA, "asf/wma format not recognized"), \


#define WMA_COMMON_ERRORS \
	PARA_ERROR(WMA_NO_GUID, "audio stream guid not found"), \


#define WMADEC_FILTER_ERRORS \
	PARA_ERROR(WMA_BAD_PARAMS, "invalid WMA parameters"), \
	PARA_ERROR(WMA_OUTPUT_SPACE, "insufficient output space"), \
	PARA_ERROR(WMA_BAD_SUPERFRAME, "invalid superframe"), \
	PARA_ERROR(WMA_BLOCK_SIZE, "invalid block size"), \
	PARA_ERROR(INCOHERENT_BLOCK_LEN, "incoherent block length"), \
	PARA_ERROR(WMADEC_EOF, "wmadec: end of file"), \


#define IMDCT_ERRORS \
	PARA_ERROR(FFT_BAD_PARAMS, "invalid params for fft"), \


#define PREBUFFER_FILTER_ERRORS \
	PARA_ERROR(PREBUFFER_SYNTAX, "syntax error in prebuffer filter config"), \
	PARA_ERROR(PREBUFFER_SUCCESS, "prebuffering complete"), \


#define OSS_WRITE_ERRORS \
	PARA_ERROR(BAD_SAMPLE_FORMAT, "sample format not supported"), \
	PARA_ERROR(BAD_CHANNEL_COUNT, "channel count not supported"), \
	PARA_ERROR(BAD_SAMPLERATE, "sample rate not supported"), \


#define AO_WRITE_ERRORS \
	PARA_ERROR(AO_DEFAULT_DRIVER, "ao: no usable output device"), \
	PARA_ERROR(AO_BAD_DRIVER, "ao: invalid driver"), \
	PARA_ERROR(AO_BAD_OPTION, "ao option is not of type key:value"), \
	PARA_ERROR(AO_APPEND_OPTION, "ao append option: memory allocation failure"), \
	PARA_ERROR(AO_OPEN_LIVE, "ao: could not open audio device"), \
	PARA_ERROR(AO_FILE_NOT_SUPP, "ao: file io drivers not supported"), \
	PARA_ERROR(AO_PLAY, "ao_play() failed"), \
	PARA_ERROR(AO_BAD_SAMPLE_FORMAT, "ao: unsigned sample formats not supported"), \
	PARA_ERROR(AO_PTHREAD, "pthread error"), \


#define COMPRESS_FILTER_ERRORS \
	PARA_ERROR(COMPRESS_SYNTAX, "syntax error in compress filter config"), \
	PARA_ERROR(COMPRESS_EOF, "compress: end of file"), \


#define WAV_FILTER_ERRORS \
	PARA_ERROR(WAV_BAD_FC, "invalid filter configuration"), \
	PARA_ERROR(WAV_EOF, "wav filter: end of file"), \
	PARA_ERROR(WAV_SUCCESS, "successfully wrote wav header"), \


#define FEC_ERRORS \
	PARA_ERROR(FEC_BAD_IDX, "invalid index vector"), \
	PARA_ERROR(FEC_SINGULAR, "unexpected singular matrix"), \
	PARA_ERROR(FEC_PIVOT, "pivot column not found"), \
	PARA_ERROR(FEC_PARMS, "invalid fec parameters"), \


#define FECDEC_FILTER_ERRORS \
	PARA_ERROR(BAD_FEC_HEADER, "invalid fec header"), \
	PARA_ERROR(BAD_SLICE_SIZE, "slice size zero or too large"), \
	PARA_ERROR(BAD_SLICE_NUM, "invalid slice number"), \
	PARA_ERROR(FECDEC_OVERRUN, "fecdec output buffer overrun"), \
	PARA_ERROR(FECDEC_EOF, "received eof packet"), \


#define AMP_FILTER_ERRORS \
	PARA_ERROR(AMP_SYNTAX, "syntax error in amp filter config"), \
	PARA_ERROR(AMP_ZERO_AMP, "no amplification necessary"), \
	PARA_ERROR(AMP_EOF, "amp: end of file"), \


#define SEND_COMMON_ERRORS \
	PARA_ERROR(MAX_CLIENTS, "maximal number of clients exceeded"), \


#define CLIENT_ERRORS \
	PARA_ERROR(TASK_STARTED, "task started"), \


#define AFH_ERRORS \
	PARA_ERROR(AFH_SYNTAX, "afh syntax error"), \
	PARA_ERROR(AFH_SHORT_WRITE, "afh short write"), \


#define AFH_COMMON_ERRORS \
	PARA_ERROR(AUDIO_FORMAT, "audio format not recognized"), \


#define ACL_ERRORS \
	PARA_ERROR(ACL_PERM, "access denied by acl"), \


#define AFS_ERRORS \
	PARA_ERROR(BAD_TABLE_NAME, "invalid table"), \
	PARA_ERROR(AFS_SYNTAX, "afs syntax error"), \
	PARA_ERROR(AFS_SIGNAL, "afs caught deadly signal"), \
	PARA_ERROR(AFS_SOCKET, "afs socket not writable"), \


#define MOOD_ERRORS \
	PARA_ERROR(NO_MOOD, "no mood available"), \
	PARA_ERROR(DUMMY_ROW, "attempted to access blob dummy object"), \


#define MM_ERRORS \
	PARA_ERROR(MOOD_SYNTAX, "mood syntax error"), \


#define ATTRIBUTE_ERRORS \
	PARA_ERROR(ATTR_SYNTAX, "attribute syntax error"), \
	PARA_ERROR(NO_ATTRIBUTES, "no attributes defined yet"), \
	PARA_ERROR(ATT_TABLE_FULL, "no more space left in attribute table"), \


#define BLOB_ERRORS \
	PARA_ERROR(BLOB_SYNTAX, "blob syntax error"), \
	PARA_ERROR(INPUT_TOO_LARGE, "input too large for stdin command"), \


#define PLAYLIST_ERRORS \
	PARA_ERROR(PLAYLIST_LOADED, ""), /* not really an error */ \
	PARA_ERROR(PATH_FOUND, ""), /* not really an error */ \
	PARA_ERROR(PLAYLIST_EMPTY, "attempted to load empty playlist"), \


#define AFT_ERRORS \
	PARA_ERROR(BAD_AFSI, "invaiid afs info"), \
	PARA_ERROR(LOCALTIME, "localtime() failed"), \
	PARA_ERROR(STRFTIME, "strftime() failed"), \
	PARA_ERROR(BAD_PATH, "invalid path"), \
	PARA_ERROR(BAD_SORT, "invalid sorting method"), \
	PARA_ERROR(FNMATCH, "fnmatch error"), \
	PARA_ERROR(NO_AFHI, "audio format handler info required"), \
	PARA_ERROR(AFT_SYNTAX, "audio file table syntax error"), \
	PARA_ERROR(HASH_MISMATCH, "hash mismatch, consider re-add"), \


#define USER_LIST_ERRORS \
	PARA_ERROR(USERLIST, "failed to open user list file"), \


#define OSX_WRITE_ERRORS \
	PARA_ERROR(STREAM_FORMAT, "could not set stream format"), \
	PARA_ERROR(ADD_CALLBACK, "can not add callback"), \
	PARA_ERROR(OPEN_COMP, "OpenAComponent() error"), \
	PARA_ERROR(UNIT_INIT, "AudioUnitInitialize() error"), \
	PARA_ERROR(UNIT_START, "AudioUnitStart() error"), \
	PARA_ERROR(DEFAULT_COMP, "can not find default audio output component"), \


#define AUDIOC_ERRORS \
	PARA_ERROR(AUDIOC_SYNTAX, "audioc syntax error"), \
	PARA_ERROR(AUDIOC_EOF, "audioc: end of file"), \


#define CLIENT_COMMON_ERRORS \
	PARA_ERROR(CLIENT_SYNTAX, "syntax error"), \
	PARA_ERROR(NO_CONFIG, "config file not found"), \
	PARA_ERROR(BAD_CONFIG, "syntax error in config file"), \
	PARA_ERROR(CLIENT_AUTH, "authentication failed"), \
	PARA_ERROR(SERVER_EOF, "connection closed by para_server"), \
	PARA_ERROR(SERVER_CMD_SUCCESS, "command terminated successfully"), \
	PARA_ERROR(SERVER_CMD_FAILURE, "command failed"), \


#define SCHED_ERRORS \
	PARA_ERROR(NOT_INITIALIZED, "scheduler not yet initialized"), \
	PARA_ERROR(SCHED_SHUTDOWN, "scheduler was shut down"), \


#define NET_ERRORS \
	PARA_ERROR(NAME_TOO_LONG, "name too long for struct sockaddr_un"), \
	PARA_ERROR(ADDRESS_LOOKUP, "can not resolve requested address"),\
	PARA_ERROR(CHMOD, "failed to set socket mode"), \
	PARA_ERROR(SENDMSG, "sendmsg() failed"), \
	PARA_ERROR(RECVMSG, "recvmsg() failed"), \
	PARA_ERROR(SCM_CREDENTIALS, "did not receive SCM credentials"), \
	PARA_ERROR(MAKESOCK, "makesock error"), \


#define UDP_RECV_ERRORS \
	PARA_ERROR(UDP_SYNTAX, "udp_recv syntax error"), \
	PARA_ERROR(UDP_OVERRUN, "output buffer overrun"), \


#define UDP_SEND_ERRORS \
	PARA_ERROR(TARGET_EXISTS, "requested target is already present"),\
	PARA_ERROR(TARGET_NOT_FOUND, "requested target not found")


#define HTTP_RECV_ERRORS \
	PARA_ERROR(HTTP_RECV_OVERRUN, "http_recv: output buffer overrun"), \


#define RECV_COMMON_ERRORS \
	PARA_ERROR(RECV_SYNTAX, "recv syntax error"), \
	PARA_ERROR(RECV_EOF, "end of file"), \


#define AUDIOD_ERRORS \
	PARA_ERROR(NO_MORE_SLOTS, "no more empty slots"), \
	PARA_ERROR(MISSING_COLON, "syntax error: missing colon"), \
	PARA_ERROR(UNSUPPORTED_AUDIO_FORMAT, "given audio format not supported"), \
	PARA_ERROR(NOT_PLAYING, "not playing"), \
	PARA_ERROR(AUDIOD_OFF, "audiod switched off"), \
	PARA_ERROR(STATUS_TIMEOUT, "timeout reading server status"), \


#define AUDIOD_COMMAND_ERRORS \
	PARA_ERROR(CLIENT_WRITE, "client write error"), \
	PARA_ERROR(UCRED_PERM, "permission denied"), \
	PARA_ERROR(INVALID_AUDIOD_CMD, "invalid command"), \
	PARA_ERROR(TOO_MANY_CLIENTS, "maximal number of stat clients exceeded"), \
	PARA_ERROR(UNKNOWN_STAT_ITEM, "status item not recognized"), \


#define FILTER_COMMON_ERRORS \
	PARA_ERROR(BAD_FILTER_OPTIONS, "invalid filter option given"), \
	PARA_ERROR(UNSUPPORTED_FILTER, "given filter not supported"), \


#define STAT_ERRORS \
	PARA_ERROR(STAT_ITEM_PARSE, "failed to parse status item"), \


#define OGGDEC_FILTER_ERRORS \
	PARA_ERROR(OGGDEC_READ, "read from media returned an error"), \
	PARA_ERROR(OGGDEC_NOTVORBIS, "bitstream is not vorbis data"), \
	PARA_ERROR(OGGDEC_VERSION, "vorbis version mismatch"), \
	PARA_ERROR(OGGDEC_BADHEADER, "invalid vorbis bitstream header"), \
	PARA_ERROR(OGGDEC_FAULT, "bug or heap/stack corruption"), \
	PARA_ERROR(OGGDEC_BADLINK, "invalid stream section or requested link corrupt"), \


#define GRAB_CLIENT_ERRORS \
	PARA_ERROR(GC_WRITE, "grab client write error"), \
	PARA_ERROR(GC_SYNTAX, "grab client syntax error"), \


#define MP3DEC_FILTER_ERRORS \
	PARA_ERROR(MAD_FRAME_DECODE, "mad frame decode error"), \
	PARA_ERROR(MP3DEC_SYNTAX, "syntax error in mp3dec config"), \
	PARA_ERROR(MP3DEC_EOF, "mp3dec: end of file"), \
	PARA_ERROR(MP3DEC_CORRUPT, "too many corrupt frames"), \


#define FILTER_ERRORS \
	PARA_ERROR(NO_FILTERS, "at least one filter must be given"), \
	PARA_ERROR(FILTER_SYNTAX, "syntax error"), \


#define STRING_ERRORS \
	PARA_ERROR(ATOI_OVERFLOW, "value too large"), \
	PARA_ERROR(STRTOLL, "unknown strtoll error"), \
	PARA_ERROR(ATOI_NO_DIGITS, "no digits found in string"), \
	PARA_ERROR(ATOI_JUNK_AT_END, "further characters after number"), \
	PARA_ERROR(SIZE_PREFIX, "bad size prefix"), \
	PARA_ERROR(REGEX, "regular expression error"), \
	PARA_ERROR(ARG_NOT_FOUND, "argument not found in arg vector"), \


#define EXEC_ERRORS \
	PARA_ERROR(DUP_PIPE, "exec error: can not create pipe"), \


#define MP3_AFH_ERRORS \
	PARA_ERROR(FRAME, "invalid mp3 frame"), \
	PARA_ERROR(FRAME_LENGTH, "invalid frame length"), \
	PARA_ERROR(MP3_INFO, "could not read mp3 info"), \
	PARA_ERROR(HEADER_FREQ, "invalid header frequency"), \
	PARA_ERROR(HEADER_BITRATE, "invalid header bitrate"), \


#define AAC_AFH_ERRORS \
	PARA_ERROR(STSZ, "did not find stcz atom"), \
	PARA_ERROR(MP4ASC, "audio spec config error"), \
	PARA_ERROR(AAC_AFH_INIT, "failed to init aac decoder"), \


#define AAC_COMMON_ERRORS \
	PARA_ERROR(ESDS, "did not find esds atom"), \
	PARA_ERROR(STCO, "did not find stco atom"), \


#define OGG_AFH_ERRORS \
	PARA_ERROR(VORBIS, "vorbis synthesis header-in error (not vorbis?)"), \
	PARA_ERROR(OGG_PACKET_IN, "ogg_stream_packetin() failed"), \
	PARA_ERROR(OGG_STREAM_FLUSH, "ogg_stream_flush() failed"), \


#define VSS_ERRORS \
	PARA_ERROR(NOFD, "did not receive open fd from afs"), \
	PARA_ERROR(BAD_CT, "invalid chunk table or bad FEC configuration"), \
	PARA_ERROR(AFS_SHORT_READ, "short read from afs socket"), \


#define CRYPT_COMMON_ERRORS \
	PARA_ERROR(SSH_KEY_HEADER, "ssh key header not found"), \
	PARA_ERROR(BASE64, "failed to base64-decode ssh public key"), \
	PARA_ERROR(KEY_PERM, "unprotected private key"), \


#define CRYPT_ERRORS \
	PARA_ERROR(PRIVATE_KEY, "can not read private key"), \
	PARA_ERROR(PUBLIC_KEY, "can not read public key"), \
	PARA_ERROR(RSA, "RSA error"), \
	PARA_ERROR(ENCRYPT, "encrypt error"), \
	PARA_ERROR(DECRYPT, "decrypt error"), \
	PARA_ERROR(BLINDING, "failed to activate key blinding"), \
	PARA_ERROR(BIGNUM, "bignum error"), \

#define GCRYPT_ERRORS \
	PARA_ERROR(MPI_SCAN, "could not scan multi-precision integer"), \
	PARA_ERROR(MPI_PRINT, "could not convert multi-precision integer"), \
	PARA_ERROR(SEXP_FIND, "could not find sublist in S-expression"), \
	PARA_ERROR(SEXP_BUILD, "could not build S-expression"), \
	PARA_ERROR(SEXP_ENCRYPT, "could not encrypt S-expression"), \
	PARA_ERROR(SEXP_DECRYPT, "could not decrypt S-expression"), \
	PARA_ERROR(MD_OPEN, "could not open message digest object"), \
	PARA_ERROR(CIPHER_OPEN, "could not create stream cipher handle"), \
	PARA_ERROR(BAD_PRIVATE_KEY, "invalid private key"), \
	PARA_ERROR(KEY_MARKER, "invalid/missing key header or footer"), \
	PARA_ERROR(ASN1_PARSE, "could not parse ASN.1 key"), \
	PARA_ERROR(SSH_PARSE, "could not parse ssh public key"), \
	PARA_ERROR(OEAP, "error during oeap (un)padding"), \


#define COMMAND_ERRORS \
	PARA_ERROR(COMMAND_SYNTAX, "syntax error in command"), \
	PARA_ERROR(AUTH_REQUEST, "did not receive auth request"), \
	PARA_ERROR(NO_AUDIO_FILE, "no audio file"), \
	PARA_ERROR(BAD_CMD, "invalid command"), \
	PARA_ERROR(PERM, "permission denied"), \
	PARA_ERROR(LOCK, "lock error"), \
	PARA_ERROR(SENDER_CMD, "command not supported by this sender"), \
	PARA_ERROR(SERVER_CRASH, "para_server crashed -- can not live without it"), \
	PARA_ERROR(BAD_USER, "auth request for invalid user"), \
	PARA_ERROR(BAD_FEATURE, "request for unknown or invalid feature"), \
	PARA_ERROR(BAD_AUTH, "authentication failure"), \


#define DCCP_RECV_ERRORS \
	PARA_ERROR(DCCP_OVERRUN, "dccp output buffer buffer overrun"), \


#define FD_ERRORS \
	PARA_ERROR(FGETS, "fgets error"), \
	PARA_ERROR(EOF, "end of file"), \
	PARA_ERROR(READ_PATTERN, "did not read expected pattern"), \
	PARA_ERROR(SHORT_WRITE, "unexpected short write"), \
	PARA_ERROR(EMPTY, "file is empty"), \


#define ALSA_WRITE_ERRORS \
	PARA_ERROR(ALSA, "alsa error"), \


#define WRITE_COMMON_ERRORS \
	PARA_ERROR(WRITE_COMMON_EOF, "end of file"), \


#define AACDEC_FILTER_ERRORS \
	PARA_ERROR(AACDEC_INIT, "failed to init aac decoder"), \
	PARA_ERROR(AAC_DECODE, "aac decode error"), \


#define CHUNK_QUEUE_ERRORS \
	PARA_ERROR(QUEUE, "packet queue overrun"), \


#define INTERACTIVE_ERRORS \
	PARA_ERROR(I9E_EOF, "end of input"), \
	PARA_ERROR(I9E_SETUPTERM, "failed to set up terminal"), \

/** \endcond errors */

/**
 * The subsystem shift.
 *
 * 255 error codes ought to be enough for every subsystem. Use the higher bits
 * of the return value to encode the subsystem number.
 */
#define SS_SHIFT 8

/**
 * Compute the subsystem offset.
 *
 * It is given by x * 2**8 where \a x is the subsystem number.
 */
#define SS_OFFSET(ss) (SS_ ## ss << SS_SHIFT)

/**
 * Make the enum of all errors of one subsystem.
 *
 * As zero should not be an error, we define a dummy enum entry with value
 * 2**ss. That lets the real errors start at 2**ss + 1.
 */
#define SS_ENUM(ss) enum {\
	E_ ## ss ## _DUMMY = SS_OFFSET(ss), \
	ss ## _ERRORS}

/**
 * Determine the subsystem number from the error number.
 *
 * Easy, it's just \a num / 2**8.
 */
#define ERRNUM_TO_SS(num) ((num) >> SS_SHIFT)

/**
 * Determine the index of an error number.
 *
 * Also easy: It's the lower 8 bits of num.
 */
#define ERRNUM_TO_INDEX(num) (((1 << SS_SHIFT) - 1) & (num))

/**
 * This bit indicates whether a number is considered a system error number
 * If yes, the system errno is just the result of clearing this bit from
 * the given number.
 */
#define SYSTEM_ERROR_BIT 30

/**
 * Like the SYSTEM_ERROR_BIT, but indicates an error from the osl library.
 */
#define OSL_ERROR_BIT 29

/** Check whether the system error bit is set. */
#define IS_SYSTEM_ERROR(num) (!!((num) & (1 << SYSTEM_ERROR_BIT)))

/** Check whether the osl error bit is set. */
#define IS_OSL_ERROR(num) (!!((num) & (1 << OSL_ERROR_BIT)))

/** Set the system error bit for the given number. */
#define ERRNO_TO_PARA_ERROR(num) ((num) | (1 << SYSTEM_ERROR_BIT))

/** Set the osl error bit for the given number. */
#define OSL_ERRNO_TO_PARA_ERROR(num) ((num) | (1 << OSL_ERROR_BIT))

/** Check whether a given number is a system error number.
 *
 * \param num The value to be checked.
 * \param _errno The system error number.
 *
 * \return True if \a num is paraslash's representation of the system
 * error identified by \a _errno.
 */
_static_inline_ int is_errno(int num, int _errno)
{
	assert(num > 0 && _errno > 0);
	return ERRNO_TO_PARA_ERROR(_errno) == num;
}

/**
 * Paraslash's version of strerror(3).
 *
 * \param num The error number.
 *
 * \return The error text of \a num.
 */
_static_inline_ const char *para_strerror(int num)
{
	assert(num > 0);
#ifdef _OSL_H
	if (IS_OSL_ERROR(num))
		return osl_strerror(num & ((1 << OSL_ERROR_BIT) - 1));
#endif
	if (IS_SYSTEM_ERROR(num))
		return strerror(num & ((1 << SYSTEM_ERROR_BIT) - 1));
	return para_errlist[ERRNUM_TO_SS(num)][ERRNUM_TO_INDEX(num)];
}

/**
 * Wrapper for osl library calls.
 *
 * \param ret The return value of an osl library function.
 *
 * This should be used for all calls to osl functions that return an osl error
 * code. It changes the return value appropriately so that it can be used for
 * printing the correct error message vi para_strerror().
 *
 * \return \a ret if \a ret >= 0, a paraslash error code otherwise.
 */
_static_inline_ int osl(int ret)
{
	if (ret >= 0)
		return ret;
	return -OSL_ERRNO_TO_PARA_ERROR(-ret);
}

/**
 * Define the error list for one subsystem.
 #
 * The first entry (index 0) is always the name of the subsystem
 * Used by macros in config.h (generated by configure).
 */
#define DEFINE_ERRLIST(ss) const char * ss ## _ERRLIST[] = {#ss, ss ## _ERRORS}

/**
 * Activate errors for one subsystem.
 *
 * Each executable needs only the error lists of those subsystems it is
 * actually linked with. We always reserve space for NUM_SS char pointers,
 * but only init those of the needed subsystems. This macro is used by macros
 * in config.h (generated by configure).
 */
#define PARA_ERRLIST(ss) [SS_ ## ss] = ss ## _ERRLIST

/**
 * This is temporarily defined to expand to its first argument (prefixed by
 * 'E_') and gets later redefined to expand to the error text only
 */
#define PARA_ERROR(err, msg) E_ ## err
#include "error2.h"
#undef PARA_ERROR
/* Rest of the world only sees the error text. */
#define PARA_ERROR(err, msg) msg
